@@ -10,10 +10,10 @@ import lucene
 warnings.simplefilter('default', DeprecationWarning)
 
 from .queries import Query, SortField, TermsFilter
-from .documents import Document, Field, MapField, NestedField, NumericField, DateTimeField
+from .documents import Document, Field, MapField, NestedField, DocValuesField, NumericField, DateTimeField
 from .indexers import TokenFilter, Analyzer, IndexSearcher, MultiSearcher, IndexWriter, Indexer, ParallelIndexer
 from .spatial import PointField, PolygonField
 
-assert lucene.VERSION >= '4.3'
-if lucene.VERSION < '4.4':
-    warnings.warn('Support for lucene 4.3 will be removed in the next release.', DeprecationWarning)
+assert lucene.VERSION >= '4.4'
+if lucene.VERSION < '4.6':
+    warnings.warn('Support for lucene 4.4 and 4.5 will be removed in the next release.', DeprecationWarning)
