@@ -5,7 +5,6 @@ Wrappers for lucene Fields and Documents.
 from future_builtins import map
 import datetime, calendar
 import operator
-import warnings
 import lucene
 from java.lang import Double, Float, Long, Number, Object
 from java.util import Arrays, HashSet
@@ -24,26 +23,7 @@ class Field(document.FieldType):
     def __init__(self, name, stored=False, indexed=True, boost=1.0, **settings):
         document.FieldType.__init__(self)
         self.name, self.boost = name, boost
-        if set(['store', 'index', 'termvector']).intersection(settings):
-            warnings.warn('lucene Field.{Store,Index,TermVector} deprecated, use FieldType attibutes', DeprecationWarning)
-            store = settings.get('store', False)
-            if isinstance(store, bool):
-                store = 'yes' if store else 'no'
-            store = document.Field.Store.valueOf(store.upper())
-            index = settings.get('index', 'analyzed')
-            if isinstance(index, bool):
-                index = document.Field.Index.toIndex(index, settings.get('analyzed', False), settings.get('omitNorms', False))
-            else:
-                index = document.Field.Index.valueOf(index.upper())
-            termvector = settings.get('termvector', False)
-            if isinstance(termvector, bool):
-                termvector = document.Field.TermVector.toTermVector(termvector, settings.get('withOffsets', False), settings.get('withPositions', False))
-            else:
-                termvector = document.Field.TermVector.valueOf(termvector.upper())
-            ft = document.Field.translateFieldType(store, index, termvector)
-            self.update(**Field.settings.fget(ft))
-        else:
-            self.update(stored=stored, indexed=indexed, **settings)
+        self.update(stored=stored, indexed=indexed, **settings)
     def update(self, docValueType='', indexOptions='', numericType='', **settings):
         if docValueType:
             self.setDocValueType(getattr(index.FieldInfo.DocValuesType, docValueType.upper()))
@@ -89,8 +69,6 @@ class NestedField(Field):
     :param sep: field separator used on name and values
     """
     def __init__(self, name, sep='.', tokenized=False, **kwargs):
-        if set(['store', 'index', 'termvector']).intersection(kwargs):
-            kwargs.setdefault('index', True)
         Field.__init__(self, name, tokenized=tokenized, **kwargs)
         self.sep = sep
         self.names = tuple(self.values(name))
@@ -113,19 +91,30 @@ class NestedField(Field):
         index = max(value.count(self.sep) for value in (start, stop) if value is not None)
         return Query.range(self.names[index], start, stop, lower, upper)
 
+class DocValuesField(Field):
+    """Field which stores a per-document values, used for efficient sorting.
+    
+    :param name: name of field
+    :param type: lucene DocValuesType string
+    """
+    def __init__(self, name, type):
+        Field.__init__(self, name, indexed=False, docValueType=type)
+        self.cls = getattr(document, type.title().replace('_', '') + 'DocValuesField')
+    def items(self, *values):
+        "Generate lucene DocValuesFields suitable for adding to a document."
+        for value in values:
+            yield self.cls(self.name, long(value) if isinstance(value, int) else util.BytesRef(value))
+
 class NumericField(Field):
     """Field which indexes numbers in a prefix tree.
     
     :param name: name of field
     :param type: optional int, float, or lucene NumericType string
     """
-    def __init__(self, name, type=None, step=0, stored=False, tokenized=False, **kwargs):
+    def __init__(self, name, type=None, tokenized=False, **kwargs):
         if type:
             kwargs['numericType'] = {int: 'long', float: 'double'}.get(type, str(type))
-        Field.__init__(self, name, stored=stored, tokenized=tokenized, **kwargs)
-        if step:
-            warnings.warn('step deprecated, use numericPrecisionStep', DeprecationWarning)
-            self.setNumericPrecisionStep(step)
+        Field.__init__(self, name, tokenized=tokenized, **kwargs)
     def items(self, *values):
         "Generate lucene NumericFields suitable for adding to a document."
         if not self.numericType():
@@ -287,8 +276,9 @@ class Hits(object):
     def items(self):
         "Generate zipped ids and scores."
         return map(operator.attrgetter('doc', 'score'), self.scoredocs)
-    def groupby(self, func):
-        "Return ordered list of `Hits`_ grouped by value of function applied to doc ids."
+    def groupby(self, func, count=None, docs=None):
+        """Return ordered list of `Hits`_ grouped by value of function applied to doc ids.
+        Optionally limit the number of groups and docs per group."""
         groups = {}
         for scoredoc in self.scoredocs:
             value = func(scoredoc.doc)
@@ -298,7 +288,11 @@ class Hits(object):
                 group = groups[value] = type(self)(self.searcher, [], fields=self.fields)
                 group.index, group.value = len(groups), value
             group.scoredocs.append(scoredoc)
-        return sorted(groups.values(), key=lambda group: group.__dict__.pop('index'))
+        groups = sorted(groups.values(), key=lambda group: group.__dict__.pop('index'))
+        for group in groups:
+            group.count, group.maxscore = len(group), max(group.scores)
+            group.scoredocs = group.scoredocs[:docs]
+        return Groups(self.searcher, groups[:count], len(groups), self.maxscore, self.fields)
     def filter(self, func):
         "Return `Hits`_ filtered by function applied to doc ids."
         scoredocs = [scoredoc for scoredoc in self.scoredocs if func(scoredoc.doc)]
@@ -308,55 +302,26 @@ class Hits(object):
         scoredocs = sorted(self.scoredocs, key=lambda scoredoc: key(scoredoc.doc), reverse=reverse)
         return type(self)(self.searcher, scoredocs, self.count, self.maxscore, self.fields)
 
-class Grouping(object):
-    """Delegated lucene SearchGroups with optimized faceting.
-    
-    :param searcher: `IndexSearcher`_ which can retrieve documents
-    :param field: unique field name to group by
-    :param query: lucene Query to select groups
-    :param count: maximum number of groups
-    :param sort: lucene Sort to order groups
-    """
-    def __init__(self, searcher, field, query=None, count=None, sort=None):
-        warnings.warn('Grouping deprecated, use GroupingSearch instead', DeprecationWarning)
-        self.searcher, self.field = searcher, field
-        self.query = query or search.MatchAllDocsQuery()
-        self.sort = sort or search.Sort.RELEVANCE
-        if count is None:
-            collector = grouping.term.TermAllGroupsCollector(field)
-            search.IndexSearcher.search(self.searcher, self.query, collector)
-            count = collector.groupCount
-        collector = grouping.term.TermFirstPassGroupingCollector(field, self.sort, count)
-        search.IndexSearcher.search(self.searcher, self.query, collector)
-        self.searchgroups = collector.getTopGroups(0, False).of_(grouping.SearchGroup)
+class Groups(object):
+    "Sequence of grouped `Hits`_."
+    select = Hits.__dict__['select']
+    def __init__(self, searcher, groupdocs, count=None, maxscore=None, fields=None):
+        self.searcher, self.groupdocs = searcher, groupdocs
+        self.count, self.maxscore = count, maxscore
+        self.fields = fields
     def __len__(self):
-        return self.searchgroups.size()
-    def __iter__(self):
-        for searchgroup in self.searchgroups:
-            yield convert(searchgroup.groupValue)
-    def facets(self, filter):
-        "Generate field values and counts which match given filter."
-        collector = grouping.term.TermSecondPassGroupingCollector(self.field, self.searchgroups, self.sort, self.sort, 1, False, False, False)
-        search.IndexSearcher.search(self.searcher, self.query, filter, collector)
-        for groupdocs in collector.getTopGroups(0).groups:
-            yield convert(groupdocs.groupValue), groupdocs.totalHits
-    def groups(self, count=1, sort=None, scores=False, maxscore=False):
-        """Generate grouped `Hits`_ from second pass grouping collector.
-        
-        :param count: maximum number of docs per group
-        :param sort: lucene Sort to order docs within group
-        :param scores: compute scores for candidate results
-        :param maxscore: compute maximum score of all results
-        """
-        sort = sort or self.sort
-        if sort == search.Sort.RELEVANCE:
-            scores = maxscore = True
-        collector = grouping.term.TermSecondPassGroupingCollector(self.field, self.searchgroups, self.sort, sort, count, scores, maxscore, True)
-        search.IndexSearcher.search(self.searcher, self.query, collector)
-        for groupdocs in collector.getTopGroups(0).groups:
-            hits = Hits(self.searcher, groupdocs.scoreDocs, groupdocs.totalHits, groupdocs.maxScore, getattr(self, 'fields', None))
+        return len(self.groupdocs)
+    def __getitem__(self, index):
+        hits = groupdocs = self.groupdocs[index]
+        if isinstance(groupdocs, grouping.GroupDocs):
+            hits = Hits(self.searcher, groupdocs.scoreDocs, groupdocs.totalHits, groupdocs.maxScore)
             hits.value = convert(groupdocs.groupValue)
-            yield hits
+        hits.fields = self.fields
+        return hits
+    @property
+    def facets(self):
+        "mapping of field values and counts"
+        return dict((hits.value, hits.count) for hits in self)
 
 class GroupingSearch(grouping.GroupingSearch):
     """Inherited lucene GroupingSearch with optimized faceting.
@@ -366,7 +331,6 @@ class GroupingSearch(grouping.GroupingSearch):
     :param cache: use unlimited caching
     :param attrs: additional attributes to set
     """
-    select = Hits.__dict__['select']
     def __init__(self, field, sort=None, cache=True, **attrs):
         grouping.GroupingSearch.__init__(self, field)
         self.field = field
@@ -381,17 +345,9 @@ class GroupingSearch(grouping.GroupingSearch):
         return self.allMatchingGroups.size()
     def __iter__(self):
         return map(convert, self.allMatchingGroups)
-    def search(self, searcher, query, filter, count):
+    def search(self, searcher, query, filter=None, count=None, start=0):
+        "Run query and return `Groups`_."
         if count is None:
             count = sum(search.FieldCache.DEFAULT.getTermsIndex(reader, self.field).valueCount for reader in searcher.readers)
-        return grouping.GroupingSearch.search(self, searcher, filter, query, 0, count).groups
-    def facets(self, searcher, filter=None):
-        "Generate field values and counts which match given filter."
-        for groupdocs in self.search(searcher, search.MatchAllDocsQuery(), filter, None):
-            yield convert(groupdocs.groupValue), groupdocs.totalHits
-    def groups(self, searcher, query, filter=None, count=None):
-        "Generate grouped `Hits`_ from search parameters."
-        for groupdocs in self.search(searcher, query, filter, count):
-            hits = Hits(searcher, groupdocs.scoreDocs, groupdocs.totalHits, groupdocs.maxScore, getattr(self, 'fields', None))
-            hits.value = convert(groupdocs.groupValue)
-            yield hits
+        topgroups = grouping.GroupingSearch.search(self, searcher, filter, query, start, count - start)
+        return Groups(searcher, topgroups.groups, topgroups.totalHitCount, topgroups.maxScore)
