@@ -6,10 +6,9 @@ The final `Indexer`_ classes exposes a high-level Searcher and Writer.
 
 from future_builtins import filter, map, zip
 import os
-import itertools, operator
+import itertools
 import contextlib
 import abc, collections
-import warnings
 import lucene
 from java.io import File, StringReader
 from java.lang import Float
@@ -18,7 +17,7 @@ from org.apache.lucene import analysis, codecs, document, index, queries, queryp
 from org.apache.pylucene.analysis import PythonAnalyzer, PythonTokenFilter
 from org.apache.pylucene.queryparser.classic import PythonQueryParser
 from .queries import Query, BooleanFilter, TermsFilter, SortField, Highlighter, FastVectorHighlighter, SpellChecker, SpellParser
-from .documents import Field, Document, Hits, Grouping, GroupingSearch
+from .documents import Field, Document, Hits, GroupingSearch
 from .spatial import DistanceComparator
 
 class Atomic(object):
@@ -214,8 +213,6 @@ class IndexReader(object):
         ids = xrange(self.maxDoc())
         bits = index.MultiFields.getLiveDocs(self.indexReader)
         return filter(bits.get, ids) if bits else iter(ids)
-    def __getitem__(self, id):
-        return Document(self.document(id))
     @property
     def directory(self):
         "reader's lucene Directory"
@@ -227,8 +224,7 @@ class IndexReader(object):
     @property
     def timestamp(self):
         "timestamp of reader's last commit"
-        directory = store.FSDirectory.cast_(self.directory).directory
-        return store.FSDirectory.fileModified(directory, self.indexCommit.segmentsFileName) * 0.001
+        return os.path.getmtime(os.path.join(self.path, self.indexCommit.segmentsFileName))
     @property
     def readers(self):
         "segment readers"
@@ -249,7 +245,7 @@ class IndexReader(object):
         copy(self.indexCommit, dest)
         with contextlib.closing(IndexWriter(dest)) as writer:
             if query:
-                writer.delete(Query(search.MatchAllDocsQuery) - query)
+                writer.delete(Query.alldocs() - query)
             if exclude:
                 writer.delete(exclude)
             writer.commit()
@@ -257,9 +253,6 @@ class IndexReader(object):
             if merge:
                 writer.forceMerge(merge)
             return len(writer)
-    def count(self, name, value):
-        "Return number of documents with given term."
-        return self.docFreq(index.Term(name, value))
     def names(self, **attrs):
         """Return field names, given option description.
         
@@ -267,27 +260,25 @@ class IndexReader(object):
         """
         fieldinfos = index.MultiFields.getMergedFieldInfos(self.indexReader).iterator()
         return [fieldinfo.name for fieldinfo in fieldinfos if all(getattr(fieldinfo, name) == attrs[name] for name in attrs)]
-    def terms(self, name, value='', stop=None, counts=False, **fuzzy):
+    def terms(self, name, value='', stop=None, counts=False, distance=0):
         """Generate a slice of term values, optionally with frequency counts.
         Supports a range of terms, wildcard terms, or fuzzy terms.
         
         :param name: field name
-        :param value: initial term text, trailing * indicates prefix
-        :param stop: optional upper bound for simple terms
+        :param value: term prefix or lower bound for range terms
+        :param stop: optional upper bound for range terms
         :param counts: include frequency counts
-        :param fuzzy: optional keyword arguments for fuzzy terms
+        :param distance: maximum edit distance for fuzzy terms
         """
-        term = index.Term(name, value)
-        args = fuzzy.get('minSimilarity', 0.5), fuzzy.get('prefixLength', 0)
         terms = index.MultiFields.getTerms(self.indexReader, name)
         if terms:
-            if fuzzy:
-                termenum = search.FuzzyTermsEnum(terms, util.AttributeSource(), term, args[0], args[1], False)
-            elif value.endswith('*'): 
-                termenum = search.PrefixTermsEnum(terms.iterator(None), util.BytesRef(value.rstrip('*')))
+            if distance:
+                termenum = search.FuzzyTermsEnum(terms, util.AttributeSource(), index.Term(name, value), float(distance), 0, False)
+            elif stop is not None:
+                termenum = search.TermRangeTermsEnum(terms.iterator(None), util.BytesRef(value), util.BytesRef(stop), True, False)
             else:
-                termenum = search.TermRangeTermsEnum(terms.iterator(None), util.BytesRef(value), stop and util.BytesRef(stop), True, False)
-            for bytesref in util.BytesRefIterator.cast_(termenum):
+                termenum = search.PrefixTermsEnum(terms.iterator(None), util.BytesRef(value))
+            for bytesref in termenum:
                 text = bytesref.utf8ToString()
                 yield (text, termenum.docFreq()) if counts else text
     def numbers(self, name, step=0, type=int, counts=False):
@@ -298,12 +289,11 @@ class IndexReader(object):
         :param type: int or float
         :param counts: include frequency counts
         """
-        term = index.Term(name, chr(ord(' ') + step))
         decode = util.NumericUtils.prefixCodedToLong
         convert = util.NumericUtils.sortableLongToDouble if issubclass(type, float) else int
         terms = index.MultiFields.getTerms(self.indexReader, name)
-        termenum = search.PrefixTermsEnum(terms.iterator(None), util.BytesRef(term.text()))
-        for bytesref in util.BytesRefIterator.cast_(termenum):
+        termenum = search.PrefixTermsEnum(terms.iterator(None), util.BytesRef(chr(ord(' ') + step)))
+        for bytesref in termenum:
             value = convert(decode(bytesref))
             yield (value, termenum.docFreq()) if counts else value
     def docs(self, name, value, counts=False):
@@ -386,7 +376,7 @@ class IndexSearcher(search.IndexSearcher, IndexReader):
         search.IndexSearcher.__init__(self, self.shared.reader(directory))
         self.analyzer = self.shared.analyzer(analyzer)
         self.filters, self.sorters, self.spellcheckers = {}, {}, {}
-        self.termsfilters, self.groupings = set(), {}
+        self.termsfilters = set()
     @classmethod
     def load(cls, directory, analyzer=None):
         "Open `IndexSearcher`_ with a lucene RAMDirectory, loading index into memory."
@@ -457,7 +447,7 @@ class IndexSearcher(search.IndexSearcher, IndexReader):
         """
         if len(query) > 1:
             return self.docFreq(index.Term(*query))
-        query = self.parse(*query, **options) if query else search.MatchAllDocsQuery()
+        query = self.parse(*query, **options) if query else Query.alldocs()
         collector = search.TotalHitCountCollector()
         search.IndexSearcher.search(self, query, options.get('filter'), collector)
         return collector.totalHits
@@ -488,7 +478,7 @@ class IndexSearcher(search.IndexSearcher, IndexReader):
         :param timeout: stop search after elapsed number of seconds
         :param parser: :meth:`Analyzer.parse` options
         """
-        query = search.MatchAllDocsQuery() if query is None else self.parse(query, **parser)
+        query = Query.alldocs() if query is None else self.parse(query, **parser)
         cache = collector = self.collector(query, count, sort, reverse, scores, maxscore)
         counter = search.TimeLimitingCollector.getGlobalCounter()
         results = collector if timeout is None else search.TimeLimitingCollector(collector, counter, long(timeout * 1000))
@@ -508,7 +498,7 @@ class IndexSearcher(search.IndexSearcher, IndexReader):
     def facets(self, query, *keys):
         """Return mapping of document counts for the intersection with each facet.
         
-        .. deprecated:: 1.5 implicit caching of filters is deprecated, in the future a `GroupingSearch`_ will be used instead
+        .. versionchanged:: 1.6 filters are no longer implicitly cached, a `GroupingSearch`_ is used instead
         
         :param query: query string, lucene Query, or lucene Filter
         :param keys: field names, term tuples, or any keys to previously cached filters
@@ -522,30 +512,20 @@ class IndexSearcher(search.IndexSearcher, IndexReader):
             query = search.CachingWrapperFilter(query)
         for key in keys:
             filters = self.filters.get(key)
-            if key in self.groupings:
-                counts[key] = dict(self.groupings[key].facets(query))
-            elif isinstance(filters, search.Filter):
+            if isinstance(filters, search.Filter):
                 counts[key] = self.count(filter=BooleanFilter.all(query, filters))
+            elif isinstance(filters, collections.Mapping):
+                for value in filters:
+                    counts[key][value] = self.count(filter=BooleanFilter.all(query, filters[value]))
+            elif isinstance(key, basestring):
+                counts[key] = self.groupby(key, Query.alldocs(), filter=query).facets
             else:
-                name, value = (key, None) if isinstance(key, basestring) else key
-                filters = self.filters.setdefault(name, {})
-                if value is None:
-                    values = filters or self.terms(name)
-                else:
-                    values = [value] if value in filters else self.terms(name, value)
-                for value in values:
-                    if value not in filters:
-                        filters[value] = Query.term(name, value).filter()
-                    counts[name][value] = self.count(filter=BooleanFilter.all(query, filters[value]))
+                name, value = key
+                counts[name][value] = self.count(filter=BooleanFilter.all(query, self.filters[name][value]))
         return dict(counts)
-    def grouping(self, field, query=None, count=None, sort=None):
-        try:
-            return self.groupings[field]
-        except KeyError:
-            return Grouping(self, field, query, count, sort)
-    def groupby(self, field, query, filter=None, count=None, **attrs):
-        "Generate `Hits`_ grouped by field using a `GroupingSearch`_."
-        return GroupingSearch(field, **attrs).groups(self, self.parse(query), filter, count)
+    def groupby(self, field, query, filter=None, count=None, start=0, **attrs):
+        "Return `Hits`_ grouped by field using a `GroupingSearch`_."
+        return GroupingSearch(field, **attrs).search(self, self.parse(query), filter, count, start)
     def sorter(self, field, type='string', parser=None, reverse=False):
         "Return `SortField`_ with cached attributes if available."
         sorter = self.sorters.get(field, SortField(field, type, parser, reverse))
@@ -575,23 +555,13 @@ class IndexSearcher(search.IndexSearcher, IndexReader):
     def suggest(self, field, prefix, count=None):
         "Return ordered suggested words for prefix."
         return self.spellchecker(field).suggest(prefix, count)
-    def correct(self, field, text, distance=2, minSimilarity=0.5):
+    def correct(self, field, text, distance=2):
         """Generate potential words ordered by increasing edit distance and decreasing frequency.
         For optimal performance only iterate the required slice size of corrections.
         
         :param distance: the maximum edit distance to consider for enumeration
-        :param minSimilarity: threshold for additional fuzzy terms after edits have been exhausted
         """
-        spellchecker = self.spellchecker(field)
-        corrections = set()
-        for words in itertools.islice(spellchecker.correct(text), distance + 1):
-            for word in words:
-                yield word
-            corrections.update(words)
-        if minSimilarity:
-            words = set(self.terms(field, text, minSimilarity=minSimilarity)) - corrections
-            for word in sorted(words, key=spellchecker.__getitem__, reverse=True):
-                yield word
+        return itertools.chain.from_iterable(itertools.islice(self.spellchecker(field).correct(text), distance + 1))
     def match(self, document, *queries):
         "Generate scores for all queries against a given document mapping."
         searcher = index.memory.MemoryIndex()
@@ -667,7 +637,7 @@ class IndexWriter(index.IndexWriter):
                 args = [codecs.Codec.getDefault()] if lucene.VERSION < '4.5' else []
                 checkindex.fixIndex(status, *args)
         finally:
-            lock.release()
+            lock.close() if hasattr(lock, 'close') else lock.release()
         return status
     def set(self, name, cls=Field, **settings):
         """Assign settings to field name.
@@ -708,16 +678,16 @@ class IndexWriter(index.IndexWriter):
         self.addIndexes([directory if isinstance(directory, store.Directory) else directory.directory])
         return self
     @contextlib.contextmanager
-    def snapshot(self, id=''):
+    def snapshot(self):
         """Return context manager of an index commit snapshot.
         
         .. versionchanged:: 1.4 lucene 4.4 identifies snapshots by commit generation
         """
-        commit = self.policy.snapshot(id) if lucene.VERSION < '4.4' else self.policy.snapshot()
+        commit = self.policy.snapshot()
         try:
             yield commit
         finally:
-            self.policy.release(id if lucene.VERSION < '4.4' else commit)
+            self.policy.release(commit)
 
 class Indexer(IndexWriter):
     """An all-purpose interface to an index.
@@ -784,7 +754,7 @@ class ParallelIndexer(Indexer):
         sorter, segments = self.sorter(self.field), self.segments
         searcher = self.indexSearcher.reopen(**caches)
         readers = [reader for reader in searcher.readers if reader.segmentName not in segments]
-        terms = set(sorter.terms(search.QueryWrapperFilter(search.MatchAllDocsQuery()), *readers))
+        terms = set(sorter.terms(Query.alldocs().filter(cache=False), *readers))
         for filter, termsfilter in self.termsfilters.items():
             if terms:
                 termsfilter.update(terms, op='andNot', cache=not self.nrt)
