@@ -2,7 +2,7 @@
 Restful json `CherryPy <http://cherrypy.org/>`_ server.
 
 The server script mounts a `WebSearcher`_ (read_only) or `WebIndexer`_ root.
-Standard `CherryPy configuration <http://docs.cherrypy.org/stable/concepts/config.html>`_ applies,
+Standard `CherryPy configuration <http://docs.cherrypy.org/en/latest/tutorial/config.html>`_ applies,
 and the provided `custom tools <#tools>`_ are also configurable.
 All request and response bodies are `application/json values <http://tools.ietf.org/html/rfc4627.html#section-2.1>`_.
 
@@ -43,20 +43,17 @@ CherryPy and Lucene VM integration issues:
 from future_builtins import map, zip
 import re
 import time
-import uuid
 import socket, httplib
 import heapq
 import collections
 import itertools
 import os, optparse
 import contextlib
-import warnings
 try:
     import simplejson as json
 except ImportError:
     import json
 import lucene
-from org.apache.lucene import search
 import cherrypy
 try:
     from . import engine, client
@@ -124,7 +121,7 @@ def validate(etag=True, last_modified=False, max_age=None, expires=None):
     root = cherrypy.request.app.root
     headers = cherrypy.response.headers
     if etag:
-        headers['etag'] = 'W/"{0}"'.format(root.searcher.version)
+        headers['etag'] = root.etag
         cherrypy.lib.cptools.validate_etags()
     if last_modified:
         headers['last-modified'] = cherrypy.lib.httputil.HTTPDate(root.searcher.timestamp)
@@ -239,14 +236,20 @@ class WebSearcher(object):
         return self
     def close(self):
         self.searcher.close()
+    @property
+    def etag(self):
+        return 'W/"{0}"'.format(self.searcher.version)
     def sync(self, host, path=''):
         "Sync with remote index."
         directory = self.searcher.path
         resource = client.Resource(host)
-        response = resource.call('PUT', '/' + '{0}/update/{1}'.format(path, uuid.uuid1()).lstrip('/'))
+        resource.headers = dict(resource.headers, **{'if-none-match': self.etag})
+        response = resource.call('PUT', '/{0}/update/snapshot'.format(path.lstrip('/')))
+        if response.status == httplib.PRECONDITION_FAILED:
+            return []
         names = sorted(set(response()).difference(os.listdir(directory)))
+        path = response.getheader('location') + '/'
         try:
-            path = response.getheader('location') + '/'
             for name in names:
                 resource.download(path + name, os.path.join(directory, name))
         finally:
@@ -344,7 +347,7 @@ class WebSearcher(object):
         return result
     @cherrypy.expose
     @cherrypy.tools.params(count=int, start=int, fields=multi, sort=multi, facets=multi, hl=multi, mlt=int, spellcheck=int, timeout=float,
-        **{'fields.multi': multi, 'fields.indexed': multi, 'facets.count': int, 'facets.min': int, 'group.count': int, 'group.limit': int, 'hl.count': int, 'mlt.fields': multi})
+        **{'fields.multi': multi, 'fields.indexed': multi, 'facets.count': int, 'facets.min': int, 'group.count': int, 'hl.count': int, 'mlt.fields': multi})
     def search(self, q=None, count=None, start=0, fields=None, sort=None, facets='', group='', hl='', mlt=None, spellcheck=0, timeout=None, **options):
         """Run query and return documents.
         
@@ -372,9 +375,10 @@ class WebSearcher(object):
                 | include facet counts for given field names; facets filters are cached
                 | optional maximum number of most populated facet values per field, and minimum count to return
             
-            &group=\ *chars*\ [:*chars*]&group.count=1&group.limit=\ *int*
+            &group=\ *chars*\ [:*chars*]&group.count=1
                 | group documents by field value with optional type, up to given maximum count
-                | limit number of groups which return docs
+            
+            .. versionchanged:: 1.6 grouping searches use count and start options
             
             &hl=\ *chars*,... &hl.count=1&hl.tag=strong&hl.enable=[fields|terms]
                 | stored fields to return highlighted
@@ -428,9 +432,20 @@ class WebSearcher(object):
         if count == 0:
             start = count = 1
         scores = options.get('sort.scores')
+        gcount = options.get('group.count', 1)
         scores = {'scores': scores is not None, 'maxscore': scores == 'max'}
-        hits = searcher.search(q, filter=qfilter, count=count, sort=sort, timeout=timeout, **scores)[start:]
-        result = {'query': q and unicode(q), 'count': hits.count, 'maxscore': hits.maxscore}
+        if ':' in group:
+            hits = searcher.search(q, filter=qfilter, sort=sort, timeout=timeout, **scores)
+            with HTTPError(httplib.BAD_REQUEST, AttributeError):
+                groups = hits.groupby(searcher.comparator(*group.split(':')).__getitem__, count=count, docs=gcount)
+            groups.groupdocs = groups.groupdocs[start:]
+        elif group:
+            scores = {'includeScores': scores['scores'], 'includeMaxScore': scores['maxscore']}
+            groups = searcher.groupby(group, q, qfilter, count, start, sort=sort, groupDocsLimit=gcount, **scores)
+        else:
+            hits = searcher.search(q, filter=qfilter, sort=sort, count=count, timeout=timeout, **scores)
+            groups = engine.documents.Groups(searcher, [hits[start:]], hits.count, hits.maxscore)
+        result = {'query': q and unicode(q), 'count': groups.count, 'maxscore': groups.maxscore}
         tag, enable = options.get('hl.tag', 'strong'), options.get('hl.enable', '')
         hlcount = options.get('hl.count', 1)
         if hl:
@@ -439,28 +454,26 @@ class WebSearcher(object):
         if fields is None:
             fields = {}
         else:
-            hits.select(*itertools.chain(fields, multi))
-        with HTTPError(httplib.BAD_REQUEST, AttributeError):
-            groups = hits.groupby(searcher.comparator(*group.split(':')).__getitem__) if group else [hits]
-        result['groups'], limit = [], options.get('group.limit', len(groups))
-        for hits in groups[:limit]:
+            groups.select(*itertools.chain(fields, multi))
+        result['groups'] = []
+        for hits in groups:
             docs = []
-            for hit in hits[:options.get('group.count', 1) if group else None]:
+            for hit in hits:
                 doc = hit.dict(*multi, **fields)
                 doc.update((name, indexed[name][hit.id]) for name in indexed)
                 fragments = (hl[name].fragments(hit.id, hlcount) for name in hl)
                 if hl:
                     doc['__highlights__'] = dict((name, value) for name, value in zip(hl, fragments) if value is not None)
                 docs.append(doc)
-            result['groups'].append({'docs': docs, 'count': len(hits), 'value': getattr(hits, 'value', None)})
-        for hits in groups[limit:]:
-            result['groups'].append({'docs': [], 'count': len(hits), 'value': hits.value})
+            result['groups'].append({'docs': docs, 'count': hits.count, 'value': getattr(hits, 'value', None)})
         if not group:
             result['docs'] = result.pop('groups')[0]['docs']
-        q = q or search.MatchAllDocsQuery()
+        q = q or engine.Query.alldocs()
         if facets:
             facets = (tuple(facet.split(':')) if ':' in facet else facet for facet in facets)
             facets = result['facets'] = searcher.facets(q, *facets)
+            for counts in facets.values():
+                counts.pop(None, None)
             if 'facets.min' in options:
                 for name, counts in facets.items():
                     facets[name] = dict((term, count) for term, count in counts.items() if count >= options['facets.min'])
@@ -474,7 +487,7 @@ class WebSearcher(object):
         return result
     @cherrypy.expose
     @cherrypy.tools.params(count=int, step=int, indexed=json.loads)
-    def terms(self, name='', value=':', *path, **options):
+    def terms(self, name='', value='*', *path, **options):
         """Return data about indexed terms.
         
         **GET** /terms?
@@ -489,12 +502,12 @@ class WebSearcher(object):
             
             :return: [*string*,... ]
         
-        **GET** /terms/*chars*/*chars*\[\*\|?\|:*chars*\|~\ *number*\]
-            Return term values (wildcards, slices, or fuzzy terms) for given field name.
+        **GET** /terms/*chars*/*chars*\[\*\|:*chars*\|~[\ *int*\]]
+            Return term values (prefix, slices, or fuzzy terms) for given field name.
             
             :return: [*string*,... ]
         
-        **GET** /terms/*chars*/*chars*\[\*\|~\]?count=\ *int*
+        **GET** /terms/*chars*/*chars*\[\*\|~[\ *int*\]\]?count=\ *int*
             Return spellchecked term values ordered by decreasing document frequency.
             Prefixes (*) are optimized to be suitable for real-time query suggestions; all terms are cached.
             
@@ -529,21 +542,19 @@ class WebSearcher(object):
                 type = getattr(__builtins__, type)
             return list(searcher.numbers(name, step=options.get('step', 0), type=type))
         if ':' in value:
-            with HTTPError(httplib.BAD_REQUEST, ValueError):
-                start, stop = value.split(':')
-            return list(searcher.terms(name, start, stop or None))
-        if 'count' in options:
-            if value.endswith('*'):
-                return searcher.suggest(name, value.rstrip('*'), options['count'])
-            if value.endswith('~'):
-                return list(itertools.islice(searcher.correct(name, value.rstrip('~')), options['count']))
-        if '*' in value or '?' in value:
+            return list(searcher.terms(name, *value.split(':')))
+        if value.endswith('*'):
+            value = value.rstrip('*')
+            if 'count' in options:
+                return searcher.suggest(name, value, options['count'])
             return list(searcher.terms(name, value))
         if '~' in value:
             with HTTPError(httplib.BAD_REQUEST, ValueError):
-                value, similarity = value.split('~')
-                similarity = float(similarity or 0.5)
-            return list(searcher.terms(name, value, minSimilarity=similarity))
+                value, distance = value.split('~')
+                distance = int(distance or 2)
+            if 'count' in options:
+                return list(itertools.islice(searcher.correct(name, value, distance), options['count']))
+            return list(searcher.terms(name, value, distance=distance))
         if not path:
             return searcher.count(name, value)
         if path[0] == 'docs':
@@ -651,14 +662,14 @@ class WebIndexer(WebSearcher):
             
             :return: *int*
         
-        **GET, PUT, DELETE** /update/*chars*
-            Verify, create, or release unique snapshot by id of current index commit and return array of referenced filenames.
+        **GET, PUT, DELETE** /update/[snapshot|*int*]
+            Verify, create, or release unique snapshot of current index commit and return array of referenced filenames.
             
             .. versionchanged:: 1.4 lucene 4.4 identifies snapshots by commit generation;  use location header
             
             :return: [*string*,... ]
         
-        **GET** /update/*chars*/*chars*
+        **GET** /update/*int*/*chars*
             Download index file corresponding to snapshot id and filename.
         """
         if not id:
@@ -667,29 +678,18 @@ class WebIndexer(WebSearcher):
             return len(self.indexer)
         method = cherrypy.request.method
         response = cherrypy.serving.response
-        if lucene.VERSION < '4.4':
-            if method == 'DELETE':
-                with HTTPError(httplib.CONFLICT, lucene.JavaError):
-                    return self.indexer.policy.release(id)
-            if method == 'PUT':
-                response.status = httplib.CREATED
-                response.headers['location'] = cherrypy.url(relative='server')
-                with HTTPError(httplib.CONFLICT, lucene.JavaError):
-                    commit = self.indexer.policy.snapshot(id)
-            else:
-                with HTTPError(httplib.NOT_FOUND, lucene.JavaError):
-                    commit = self.indexer.policy.getSnapshot(id)
+        if method == 'PUT':
+            if id != 'snapshot':
+                raise cherrypy.NotFound()
+            commit = self.indexer.policy.snapshot()
+            response.status = httplib.CREATED
+            response.headers['location'] = cherrypy.url('/update/{0:d}'.format(commit.generation), relative='server')
         else:
-            if method == 'PUT':
-                commit = self.indexer.policy.snapshot()
-                response.status = httplib.CREATED
-                response.headers['location'] = cherrypy.url('/update/{0:d}'.format(commit.generation), relative='server')
-            else:
-                with HTTPError(httplib.NOT_FOUND, ValueError, AssertionError):
-                    commit = self.indexer.policy.getIndexCommit(long(id))
-                    assert commit is not None, 'commit not snapshotted'
-                if method == 'DELETE':
-                    self.indexer.policy.release(commit)
+            with HTTPError(httplib.NOT_FOUND, ValueError, AssertionError):
+                commit = self.indexer.policy.getIndexCommit(long(id))
+                assert commit is not None, 'commit not snapshotted'
+            if method == 'DELETE':
+                self.indexer.policy.release(commit)
         if not name:
             return list(commit.fileNames)
         with HTTPError(httplib.NOT_FOUND, TypeError, AssertionError):
@@ -761,7 +761,7 @@ class WebIndexer(WebSearcher):
             
             {"stored"|"indexed"\|...: *string*\|true|false,... }
             
-            .. deprecated:: 1.5 lucene FieldType attributes used as settings
+            .. versionchanged:: 1.6 lucene FieldType attributes used as settings
             
             :return: {"stored"|"indexed"\|...: *string*\|true|false,... }
         """
@@ -771,10 +771,7 @@ class WebIndexer(WebSearcher):
             if name not in self.indexer.fields:
                 cherrypy.response.status = httplib.CREATED
             with HTTPError(httplib.BAD_REQUEST, AttributeError):
-                with warnings.catch_warnings(record=True) as messages:
-                    self.indexer.set(name, **settings)
-            for message in messages:
-                cherrypy.response.headers['warning'] = '199 lupyne "{0}"'.format(message.message)
+                self.indexer.set(name, **settings)
         with HTTPError(httplib.NOT_FOUND, KeyError):
             return self.indexer.fields[name].settings
 
