@@ -11,7 +11,7 @@ import contextlib
 import lucene
 from java.io import StringReader
 from org.apache.lucene import analysis, document, search, store, util
-from org.apache.lucene.search import grouping, highlight, vectorhighlight
+from org.apache.lucene.search import highlight, vectorhighlight
 from org.apache.pylucene.search import PythonFilter
 from lupyne import engine
 from . import fixture
@@ -120,6 +120,9 @@ class TestCase(BaseTest):
         query &= engine.Query.fuzzy('text', 'hello')
         query |= engine.Query.fuzzy('text', 'hello', 1)
         assert str(query) == '+text:* +text:hello~2 text:hello~1'
+        query = engine.Query.regexp('text', '.*')
+        assert str(query) == 'text:/.*/'
+        assert str(query.constant()) == 'ConstantScore(text:/.*/)'
         query = engine.Query.span('text', 'world')
         assert str(query.mask('name')) == 'mask(text:world) as name'
         assert str(query.payload()) == 'spanPayCheck(text:world, payloadRef: )'
@@ -153,23 +156,20 @@ class TestCase(BaseTest):
         assert [indexer[id].dict() for id in indexer] == [{'name': 'new'}]
         indexer.deleteAll()
         indexer.commit()
-        temp = engine.Indexer(self.tempdir)
-        temp.add()
-        temp.commit()
-        indexer += temp
-        indexer += temp.directory
-        indexer += self.tempdir
+        with contextlib.closing(engine.Indexer(self.tempdir)) as temp:
+            temp.add()
+        for other in (temp, temp.directory, self.tempdir):
+            indexer += other
         assert len(indexer) == 3
         indexer.add(text=analysis.core.WhitespaceTokenizer(util.Version.LUCENE_CURRENT, StringReader('?')), name=util.BytesRef('{}'))
         indexer.commit()
         assert indexer[next(indexer.docs('text', '?'))]['name'] == '{}'
-        reader = engine.indexers.IndexReader(indexer.indexReader)
-        assert reader[0].dict() == {} and reader.count('text', '?') == 1
         indexer.delete('text', '?')
         indexer.commit(merge=True)
         assert not indexer.hasDeletions()
         indexer.commit(merge=1)
         assert len(list(indexer.readers)) == 1
+        reader = engine.indexers.IndexReader(indexer.indexReader)
         del reader.indexReader
         self.assertRaises(AttributeError, getattr, reader, 'maxDoc')
         del indexer.indexSearcher
@@ -188,15 +188,15 @@ class TestCase(BaseTest):
         engine.IndexSearcher.load(searcher.directory) # ensure directory isn't closed
         assert len(indexer) == len(searcher) and store.RAMDirectory.instance_(searcher.directory)
         assert indexer.filters == indexer.spellcheckers == {}
-        assert indexer.facets(search.MatchAllDocsQuery(), 'amendment')
+        assert indexer.facets(engine.Query.alldocs(), 'amendment')
         assert indexer.suggest('amendment', '')
-        assert list(indexer.filters) == list(indexer.spellcheckers) == ['amendment']
+        assert list(indexer.spellcheckers) == ['amendment']
         indexer.delete('amendment', doc['amendment'])
         indexer.add(doc)
         reader = indexer.indexReader
         indexer.commit(filters=True, spellcheckers=True)
         assert reader.refCount == 0
-        assert list(indexer.filters) == list(indexer.spellcheckers) == ['amendment']
+        assert list(indexer.spellcheckers) == ['amendment']
         tokenizer = lambda reader: analysis.core.WhitespaceTokenizer(util.Version.LUCENE_CURRENT, reader)
         doc['amendment'] = engine.Analyzer(tokenizer).tokens(doc['amendment'])
         doc['date'] = engine.Analyzer(tokenizer).tokens(doc['date']), 2.0
@@ -209,7 +209,8 @@ class TestCase(BaseTest):
         comparator = searcher.comparator('amendment')
         assert set(map(type, comparator)) == set([int])
         assert searcher is searcher.reopen()
-        assert searcher.facets(search.MatchAllDocsQuery(), 'amendment')['amendment'] == dict.fromkeys(map(str, range(1, 28)), 2)
+        facets = searcher.facets(engine.Query.alldocs(), 'amendment')['amendment']
+        assert facets.pop(None) == 16 and facets == dict.fromkeys(map(str, range(1, 28)), 2)
         reader = searcher.indexReader
         del searcher
         assert not reader.refCount
@@ -220,8 +221,9 @@ class TestCase(BaseTest):
         assert sorted(map(int, articles)) == range(1, 8)
         assert sorted(map(int, indexer.terms('amendment'))) == range(1, 28)
         assert list(itertools.islice(indexer.terms('text', 'right'), 2)) == ['right', 'rights']
-        assert list(indexer.terms('text', 'right*')) == ['right', 'rights']
-        assert list(indexer.terms('text', 'right', minSimilarity=0.5)) == ['eight', 'right', 'rights']
+        assert list(indexer.terms('text', 'right')) == ['right', 'rights']
+        assert list(indexer.terms('text', 'right', distance=1)) == ['eight', 'right', 'rights']
+        assert list(indexer.terms('text', 'senite', distance=2)) == ['senate', 'sent']
         word, count = next(indexer.terms('text', 'people', counts=True))
         assert word == 'people' and count == 8
         docs = dict(indexer.docs('text', 'people', counts=True))
@@ -336,14 +338,10 @@ class TestCase(BaseTest):
         assert indexer.suggest('text', 'con')[:2] == ['congress', 'constitution']
         assert indexer.suggest('text', 'congress') == ['congress']
         assert indexer.suggest('text', 'congresses') == []
-        assert list(indexer.correct('text', 'writ', distance=0, minSimilarity=None)) == ['writ']
-        assert list(indexer.correct('text', 'write', distance=0, minSimilarity=None)) == []
-        assert list(indexer.correct('text', 'write', distance=0)) == ['crime', 'writs', 'written', 'writ']
-        assert list(indexer.correct('text', 'write', distance=0, minSimilarity=0.7)) == ['writs', 'writ']
-        assert list(indexer.correct('text', 'write', distance=1, minSimilarity=None)) == ['writs', 'writ']
-        assert list(indexer.correct('text', 'write', distance=1)) == ['writs', 'writ', 'crime', 'written']
-        assert list(indexer.correct('text', 'write', distance=1, minSimilarity=0.7)) == ['writs', 'writ']
-        assert list(indexer.correct('text', 'write', minSimilarity=0.9)) == ['writs', 'writ', 'crime', 'written']
+        assert list(indexer.correct('text', 'writ', distance=0)) == ['writ']
+        assert list(indexer.correct('text', 'write', distance=0)) == []
+        assert list(indexer.correct('text', 'write', distance=1)) == ['writs', 'writ']
+        assert list(indexer.correct('text', 'write')) == ['writs', 'writ', 'crime', 'written']
         query = indexer.parse('text:write', spellcheck=True)
         assert search.TermQuery.instance_(query) and str(query) == 'text:writs'
         query = indexer.parse('"hello world"', field='text', spellcheck=True)
@@ -351,7 +349,7 @@ class TestCase(BaseTest):
         assert str(indexer.parse('vwxyz', field='text', spellcheck=True)) == 'text:vwxyz'
         files = set(os.listdir(self.tempdir))
         path = os.path.join(self.tempdir, 'temp')
-        with indexer.snapshot('backup') as commit:
+        with indexer.snapshot() as commit:
             indexer.commit(merge=1)
             assert indexer.indexCommit.generation > commit.generation
             engine.indexers.copy(commit, path)
@@ -400,34 +398,19 @@ class TestCase(BaseTest):
         la, orange = sorted(filter(facets.get, facets))
         assert la == 'CA.Los Angeles' and facets[la] > 100
         assert orange == 'CA.Orange' and facets[orange] > 10
-        (field, facets), = indexer.facets(query, ('state.county', 'CA.*')).items()
-        assert all(value.startswith('CA.') for value in facets) and set(facets) < set(indexer.filters[field])
-        with assertWarns(DeprecationWarning):
-            assert set(indexer.grouping('state', count=1)) < set(indexer.grouping('state')) == set(states)
-        grouper = indexer.grouping(field, query, sort=search.Sort(indexer.sorter(field)))
-        assert len(grouper) == 2 and list(grouper) == [la, orange]
-        for value, (name, count) in zip(grouper, grouper.facets(None)):
-            assert value == name and count > 0
-        grouper = indexer.groupings[field] = indexer.grouping(field, engine.Query.term('state', 'CA'))
-        assert indexer.facets(query, field)[field] == facets
-        hits = next(grouper.groups())
-        assert hits.value == 'CA.Los Angeles' and hits.count > 100 and len(hits) == 1
-        hit, = hits
-        assert hit.score in hit.keys
-        assert hit['county'] == 'Los Angeles' and hits.maxscore >= hit.score > 0
-        hits = next(grouper.groups(count=2, sort=search.Sort(indexer.sorter('zipcode')), scores=True))
-        assert hits.value == 'CA.Los Angeles' and math.isnan(hits.maxscore) and len(hits) == 2
-        assert all(hit.score > 0 and hit['zipcode'] > '90000' and hit['zipcode'] in hit.keys for hit in hits)
-        hits, = indexer.groupby(field, engine.Query.term('state', 'CA'), count=1)
+        indexer.filters[field] = dict((term, engine.Query.term(field, term).filter()) for term in indexer.terms(field, 'CA.'))
+        (field, facets), = indexer.facets(query, field).items()
+        assert all(value.startswith('CA.') for value in facets) and set(facets) == set(indexer.filters[field])
+        (field, facets), = indexer.facets(query, (field, 'CA.Los Angeles')).items()
+        assert facets == {'CA.Los Angeles': 264}
+        groups = indexer.groupby(field, engine.Query.term('state', 'CA'), count=1)
+        assert len(groups) == 1 < groups.count
+        hits, = groups
         assert hits.value == 'CA.Los Angeles' and len(hits) == 1 and hits.count > 100
         grouping = engine.documents.GroupingSearch(field, sort=search.Sort(indexer.sorter(field)), cache=False, allGroups=True)
-        assert all(dict(grouping.facets(indexer.indexSearcher)).values())
+        assert all(grouping.search(indexer.indexSearcher, engine.Query.alldocs()).facets.values())
         assert len(grouping) == len(list(grouping)) > 100
         assert set(grouping) > set(facets)
-        grouping.select()
-        hits, = grouping.groups(indexer.indexSearcher, engine.Query.term('state', 'CA'), count=1)
-        assert hits.value == 'CA.Alameda' and len(hits) == 1 and hits.count > 50
-        assert hits[0] == {}
         for count in (None, len(indexer)):
             hits = indexer.search(query, count=count, timeout=0.01)
             assert 0 <= len(hits) <= indexer.count(query) and hits.count in (None, len(hits)) and hits.maxscore in (None, 1.0)
@@ -506,16 +489,17 @@ class TestCase(BaseTest):
         settings = {'indexed': False, 'docValueType': 'NUMERIC', 'indexOptions': 'DOCS_ONLY'}
         field = engine.Field('', **settings)
         assert field.settings == engine.Field('', **field.settings).settings == settings
-        with assertWarns(DeprecationWarning):
-            field = engine.NestedField('', store=True)
+        field = engine.NestedField('', stored=True)
         assert field.settings == {'indexed': True, 'tokenized': False, 'stored': True}
         field = engine.Field('', omitNorms=True, storeTermVectors=True, storeTermVectorPositions=True, storeTermVectorOffsets=True)
         field, = field.items(' ')
         attrs = 'indexed', 'tokenized', 'storeTermVectors', 'storeTermVectorPositions', 'storeTermVectorOffsets', 'omitNorms'
         assert all(getattr(field.fieldType(), attr)() for attr in attrs)
+        field, = engine.DocValuesField('', 'sorted').items('')
+        assert str(field.fieldType().docValueType()) == 'SORTED' and field.binaryValue().utf8ToString() == ''
+        field, = engine.DocValuesField('', 'numeric').items(0)
+        assert str(field.fieldType().docValueType()) == 'NUMERIC' and field.numericValue().longValue() == 0
         indexer = engine.Indexer(self.tempdir)
-        indexer.set('', store='yes', termvector='no')
-        assert indexer.fields.pop('').settings == {'indexed': True, 'stored': True}
         indexer.set('amendment', engine.MapField, func='{0:02d}'.format, stored=True)
         indexer.set('size', engine.MapField, func='{0:04d}'.format, stored=True)
         field = indexer.fields['date'] = engine.NestedField('Y-m-d', sep='-', stored=True)
@@ -573,8 +557,7 @@ class TestCase(BaseTest):
         indexer = engine.Indexer(self.tempdir)
         indexer.set('amendment', engine.NumericField, type=int, stored=True)
         indexer.set('date', engine.DateTimeField, stored=True)
-        with assertWarns(DeprecationWarning):
-            indexer.set('size', engine.NumericField, type=int, stored=True, step=5)
+        indexer.set('size', engine.NumericField, type=int, stored=True, numericPrecisionStep=5)
         for doc in fixture.constitution.docs():
             if 'amendment' in doc:
                 indexer.add(amendment=int(doc['amendment']), date=[tuple(map(int, doc['date'].split('-')))], size=len(doc['text']))
