@@ -1,6 +1,7 @@
 from future_builtins import map
 import unittest
 import os, sys
+import tempfile, shutil
 import signal, subprocess
 import operator
 import httplib
@@ -134,8 +135,7 @@ class TestCase(BaseTest):
         assert response.status == httplib.CREATED and response() == defaults
         response = resource.call('PUT', '/fields/text', {})
         assert response.status == httplib.OK and response() == defaults
-        with local.assertWarns(UserWarning):
-            assert resource.put('/fields/name', {'store': True, 'index': 'not_analyzed'})
+        assert resource.put('/fields/name', {'stored': True, 'tokenized': False})
         assert sorted(resource.get('/fields')) == ['name', 'text']
         assert resource.get('/fields/text') == {'indexed': True}
         assert not resource.post('/docs', [{'name': 'sample', 'text': 'hello world'}])
@@ -247,7 +247,10 @@ class TestCase(BaseTest):
         assert not resource.delete('/search')
         responses = list(resource.multicall(('POST', '/docs', [{}]), ('POST', '/update'), ('GET', '/docs')))
         assert responses[0].status == httplib.ACCEPTED and responses[1]() == len(responses[2]()) == 1
-        assert resource.post('/', [self.tempdir]).values() == [2]
+        tempdir = tempfile.mkdtemp(dir=os.path.dirname(__file__))
+        engine.IndexWriter(tempdir).add()
+        assert resource.post('/', [tempdir]).values() == [2]
+        shutil.rmtree(tempdir)
         with local.assertWarns(DeprecationWarning, UserWarning):
             assert Resource(resource.host, resource.port).call('GET', '/missing', redirect=True)()
         with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
@@ -266,13 +269,8 @@ class TestCase(BaseTest):
             response = resource.call('GET', path + '/' + name)
             assert response and response.getheader('content-type') == 'application/x-download'
             assert len(response.body) == os.path.getsize(os.path.join(self.tempdir, name))
-        assert resource.put('/update/backup') == names
-        if lucene.VERSION < '4.4':
-            with assertRaises(httplib.HTTPException, httplib.CONFLICT):
-                resource.put('/update/snapshot')
-            assert not resource.delete('/update/snapshot')
-            with assertRaises(httplib.HTTPException, httplib.CONFLICT):
-                resource.delete('/update/snapshot')
+        with assertRaises(httplib.HTTPException, httplib.NOT_FOUND):
+            assert resource.put('/update/backup') == names
         resource = client.Resource('localhost', self.ports[-1] + 1)
         with assertRaises(socket.error, errno.ECONNREFUSED):
             resource.get('/')
@@ -314,13 +312,13 @@ class TestCase(BaseTest):
         assert sorted(map(int, articles)) == range(1, 8)
         assert sorted(map(int, resource.get('/terms/amendment'))) == range(1, 28)
         assert resource.get('/terms/text/:0') == []
-        assert resource.get('/terms/text/z:') == []
         assert resource.get('/terms/text/right:right~') == resource.get('/terms/text/right*') == ['right', 'rights']
         assert resource.get('/terms/text/writ*') == ['writ', 'writing', 'writings', 'writs', 'written']
         assert resource.get('/terms/text/*?count=0') == []
         assert resource.get('/terms/text/writ*?count=10') == ['writs', 'writ', 'writing', 'writings', 'written']
         assert resource.get('/terms/text/writ*?count=3') == ['writs', 'writ', 'writing']
-        assert resource.get('/terms/text/right~') == resource.get('/terms/text/right~0.5') == ['eight', 'right', 'rights']
+        assert resource.get('/terms/text/right~1') == ['eight', 'right', 'rights']
+        assert resource.get('/terms/text/right~') == ['eight', 'high', 'right', 'rights']
         assert resource.get('/terms/text/right~?count=3') == ['right', 'eight', 'rights']
         assert resource.get('/terms/text/right~?count=5') == ['right', 'eight', 'rights', 'high']
         assert resource.get('/terms/text/write~?count=5') == ['writs', 'writ', 'crime', 'written']
@@ -374,8 +372,8 @@ class TestCase(BaseTest):
         assert result['count'] == sum(sum(facets.values()) for facets in result['facets'].values())
         for name, keys in [('article', ['1', 'Preamble']), ('amendment', ['1', '10', '17', '2', '4', '9'])]:
             assert sorted(key for key, value in result['facets'][name].items() if value) == keys
-        result = resource.get('/search', q='text:president', facets='date:19*')
-        assert all(key.startswith('19') and value in (0, 1) for key, value in result['facets']['date'].items())
+        result = resource.get('/search', q='text:president', facets='date')
+        assert len(result['facets']['date']) == sum(result['facets']['date'].values()) == 7
         result = resource.get('/search', q='text:freedom')
         assert result['count'] == 1
         doc, = result['docs']
@@ -412,13 +410,13 @@ class TestCase(BaseTest):
         assert result['count'] == 8 and set(doc['__score__'] for doc in result['docs']) == set([1.0])
         result = resource.get('/search', q='text:right', filter='text:people')
         assert result['count'] == 4 and 0 < result['maxscore'] < 1.0
-        result = resource.get('/search', q='text:right', group='date', **{'group.count': 2, 'group.limit': 2})
-        assert 'docs' not in result and len(result['groups']) == 9
-        assert sum(map(operator.itemgetter('count'), result['groups'])) == result['count'] == 13
+        result = resource.get('/search', q='text:right', group='date', count=2, **{'group.count': 2})
+        assert 'docs' not in result and len(result['groups']) == 2
+        assert sum(map(operator.itemgetter('count'), result['groups'])) < result['count'] == 13
         assert all(min(group['count'], 2) >= len(group['docs']) for group in result['groups'])
         assert all(doc.get('date') == group['value'] for group in result['groups'] for doc in group['docs'])
         group = result['groups'][0]
-        assert group['value'] == '1791-12-15' and not result['groups'][-1]['value']
+        assert group['value'] == '1791-12-15'
         assert sorted(group) == ['count', 'docs', 'value'] and group['count'] == 5
         assert len(group['docs']) == 2 and group['docs'][0]['amendment'] == '2'
         assert len(result['groups'][1]['docs']) == 1 and all(group['docs'] == [] for group in result['groups'][2:])
@@ -447,14 +445,14 @@ class TestCase(BaseTest):
         result = resource.get('/search', count=0, facets='county')
         facets = result['facets']['county']
         assert result['count'] == sum(facets.values()) and 'Los Angeles' in facets
-        result = resource.get('/search', count=0, facets='county.city:Los Angeles.*')
+        result = resource.get('/search', q='Los Angeles', count=0, facets='county.city', **{'q.type': 'term', 'q.field': 'county'})
         facets = result['facets']['county.city']
-        assert result['count'] > sum(facets.values()) and all(location.startswith('Los Angeles.') for location in facets)
+        assert result['count'] == sum(facets.values()) and all(location.startswith('Los Angeles.') for location in facets)
         result = resource.get('/search', count=0, facets='county', **{'facets.count': 3})
         assert sorted(result['facets']['county']) == ['Los Angeles', 'Orange', 'San Diego']
         result = resource.get('/search', count=0, facets='county', **{'facets.min': 140})
         assert sorted(result['facets']['county']) == ['Los Angeles', 'Orange', 'San Diego']
-        result = resource.get('/search', q='Los Angeles', group='county.city', **{'group.count': 0, 'q.field': 'county', 'q.type': 'prefix'})
+        result = resource.get('/search', q='Los Angeles', group='county.city', **{'group.count': 2, 'q.field': 'county', 'q.type': 'prefix'})
         assert all(group['value'].startswith('Los Angeles') for group in result['groups'])
         assert sum(map(operator.itemgetter('count'), result['groups'])) == sum(facets.values()) == result['count']
         assert resource.get('/queries') == []
